function model = train_protein_model( imgdir, param )
% Train objectized protein and position model using 3D HeLa GFP images

% Author: Tao Peng
% Edited: Ivan E. Cao-Berg
%
% Copyright (C) 2011-2012 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% April 17, 2012 I. Cao-Berg Added parameter structure to method
% April 24, 2012 R.F. Murphy Correction Gaussian object frequency model
% June 19, 2012 I. Cao-Berg Uncommented training of the frequency model
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

if nargin == 1
  param = [];
  verbose = false;
  debug = false;
elseif nargin == 2
  try 
   verbose = param.verbose;
   if ~islogical( verbose )
    verbose = false;
   end
  catch
   verbose = false;
  end

  try
   debug = param.debug;
   if ~islogical( debug )
    debug = false;
   end
  catch
    debug = false;
  end
else
  error('CellOrganizer: Wrong number of input arguments.');
end

temporaryFilesFolder = [ pwd filesep 'temp' filesep 'protein_objects_gaussian' ];
if ~exist( temporaryFilesFolder, 'dir' )
    mkdir( temporaryFilesFolder );
end

% Extract protein objects from 4 image sets (note this step can be parallelized)
savedir = [ temporaryFilesFolder filesep 'original_objects'];
if ~exist( savedir, 'dir' )
    mkdir( savedir );
end

disp( 'Finding objects in images...' );
tic;
findobjs_run( imgdir, savedir );
toc

disp( 'Fitting Gaussian mixtures to the objects...' );
savedir = './temp/protein_objects_gaussian/object_gaussians';
if ~exist(savedir,'dir')
    mkdir(savedir);
end
 
tic;
learngaussobjs_run2(imgdir,savedir);
toc

disp( 'Learning object models...' );
gaussianobjdir = './temp/protein_objects_gaussian/object_gaussians';
savedir = './temp/protein_objects_gaussian/object_stats';
if ~exist(savedir,'dir')
    mkdir(savedir);
end

tic;
gmm_objempdistr(gaussianobjdir,savedir);
model.size = gmm_objsizefit(savedir);
toc

disp( 'Learning protein frequency model...' );
savedir = './temp/protein_objects_gaussian/object_gaussians';
if ~exist(savedir,'dir')
    mkdir(savedir);
end

tic;
disp( 'Learning frequency model...' );
model.frequency = getProteinFrequencyModel( savedir );
toc 

disp( 'Learning object position models...' );
savedir = './temp/protein_objects_gaussian/object_positions';
if ~exist(savedir,'dir')
    mkdir(savedir)
end

tic;
model.position.beta = objpos_run(imgdir,savedir);
toc 

% Save protein models
% disp( 'Saving protein models' );
% savedir = './inter_results/protein_objects_gaussian';
% save([savedir filesep 'model'],'model');
